'use strict';

const assert = require('assert');
const Service = require('egg').Service;
class DrafttService extends Service {
  constructor(ctx) {
    super(ctx);
    this.model = this.ctx.model.Draft;
  }
  async create({ title, name, phone, workUnit, address, url, openid, status }) {
    assert(title, '标题不存在');
    assert(name, '姓名不存在');
    assert(phone, '电话不存在');
    assert(workUnit, '单位不存在');
    assert(address, '地址不存在');
    assert(openid, '用户ID不存在');
    assert(status, '状态不存在');
    try {
      const res = await this.model.create({ title, name, phone, workUnit, address, url, openid, status });
      return { errcode: 0, errmsg: 'ok', data: res };
    } catch (error) {
      throw error;
    }
  }
  async update({ id, title, name, phone, workUnit, address, url, openid, status }) {
    assert(id, 'id不存在');
    try {
      await this.model.updateOne({ _id: id }, { title, name, phone, workUnit, address, url, openid, status });
      return { errcode: 0, errmsg: 'ok', data: '' };
    } catch (error) {
      throw error;
    }
  }
  async delete({ id }) {
    assert(id, 'id不存在');
    try {
      await this.model.deleteOne({ _id: id });
      return { errcode: 0, errmsg: 'ok', data: '' };
    } catch (error) {
      throw error;
    }
  }
  async query({ skip, limit, title, name, phone, status }) {
    const filter = {};
    const arr = { name, title, phone, status };
    for (const e in arr) {
      const data = `{ "${e}": { "$regex": "${arr[e]}" } }`;
      if (arr[e]) {
        filter.$or = [];
        filter.$or.push(JSON.parse(data));
      }
    }
    try {
      const total = await this.model.find({ ...filter });
      let res;
      if (skip && limit) {
        res = await this.model.find({ ...filter }).skip(Number(skip) * Number(limit)).limit(Number(limit));
      } else {
        res = await this.model.find({ ...filter });
      }
      return { errcode: 0, errmsg: 'ok', data: res, total: total.length };
    } catch (error) {
      throw error;
    }
  }
}

module.exports = DrafttService;
