'use strict';

const assert = require('assert');
const Service = require('egg').Service;
class PowerService extends Service {
  constructor(ctx) {
    super(ctx);
    this.model = this.ctx.model.Power;
  }
  async create({ vipTab, openid, list }) {
    assert(list, 'ID列表不存在');
    assert(openid, 'openid不存在');
    assert(vipTab, '会员标识不存在');
    try {
      const res = await this.model.create({ vipTab, openid, list });
      return { errcode: 0, errmsg: 'ok', data: res };
    } catch (error) {
      throw error;
    }
  }
  async update({ id, vipTab, list }) {
    assert(id, 'id不存在');
    try {
      await this.model.updateOne({ _id: id }, { vipTab, list });
      return { errcode: 0, errmsg: 'ok', data: '' };
    } catch (error) {
      throw error;
    }
  }
  async delete({ id }) {
    assert(id, 'id不存在');
    try {
      await this.model.deleteOne({ _id: id });
      return { errcode: 0, errmsg: 'ok', data: '' };
    } catch (error) {
      throw error;
    }
  }
  async query({ skip, limit, vipTab, openid, list }) {
    const filter = {};
    const arr = { vipTab, openid, list };
    for (const e in arr) {
      let datas;
      if (e === 'list') {
        datas = `{ "${e}": { "$elemMatch": { "$eq": "${arr[e]}" } } }`;
      } else {
        datas = `{ "${e}": { "$regex": "${arr[e]}" } }`;
      }
      if (arr[e]) {
        filter.$or = [];
        filter.$or.push(JSON.parse(datas));
      }
    }
    try {
      const total = await this.model.find({ ...filter });
      let res;
      if (skip && limit) {
        res = await this.model.find({ ...filter }).skip(Number(skip) * Number(limit)).limit(Number(limit));
      } else {
        res = await this.model.find({ ...filter });
      }
      return { errcode: 0, errmsg: 'ok', data: res, total: total.length };
    } catch (error) {
      throw error;
    }
  }
}

module.exports = PowerService;
