'use strict';

const assert = require('assert');
const Service = require('egg').Service;
class MarketService extends Service {
  constructor(ctx) {
    super(ctx);
    this.model = this.ctx.model.Market;
  }
  async create({ name, type, bindId, status, money, integral }) {
    assert(name, '商品名称不存在');
    assert(type, '商品类型不存在');
    assert(bindId, '货品ID不存在');
    assert(status, '商品状态不存在');
    assert(money, '价格不存在');
    try {
      const res = await this.model.create({ name, type, bindId, status, money, integral });
      return { errcode: 0, errmsg: 'ok', data: res };
    } catch (error) {
      throw error;
    }
  }
  async update({ id, name, type, bindId, status, money, integral }) {
    assert(id, 'id不存在');
    try {
      await this.model.updateOne({ _id: id }, { name, type, bindId, status, money, integral });
      return { errcode: 0, errmsg: 'ok', data: '' };
    } catch (error) {
      throw error;
    }
  }
  async delete({ id }) {
    assert(id, 'id不存在');
    try {
      await this.model.deleteOne({ _id: id });
      return { errcode: 0, errmsg: 'ok', data: '' };
    } catch (error) {
      throw error;
    }
  }
  async query({ skip, limit, name, type, bindId, status, money, integral }) {
    const filter = {};
    const arr = { name, type, bindId, status, money, integral };
    for (const e in arr) {
      const data = `{ "${e}": { "$regex": "${arr[e]}" } }`;
      if (arr[e]) {
        filter.$or = [];
        filter.$or.push(JSON.parse(data));
      }
    }
    try {
      const total = await this.model.find({ ...filter });
      let res;
      if (skip && limit) {
        res = await this.model.find({ ...filter }).skip(Number(skip) * Number(limit)).limit(Number(limit));
      } else {
        res = await this.model.find({ ...filter });
      }
      return { errcode: 0, errmsg: 'ok', data: res, total: total.length };
    } catch (error) {
      throw error;
    }
  }
}

module.exports = MarketService;
