'use strict';

const assert = require('assert');
const Service = require('egg').Service;
class OrgService extends Service {
  constructor(ctx) {
    super(ctx);
    this.model = this.ctx.model.Org;
  }
  async create({ name, code }) {
    assert(name, '名称不存在');
    assert(code, '编码不存在');
    try {
      const res = await this.model.create({ name, code });
      return { errcode: 0, errmsg: 'ok', data: res };
    } catch (error) {
      throw error;
    }
  }
  async update({ id, name, code }) {
    assert(id, 'id不存在');
    try {
      await this.model.updateOne({ _id: id }, { name, code });
      return { errcode: 0, errmsg: 'ok', data: '' };
    } catch (error) {
      throw error;
    }
  }
  async delete({ id }) {
    assert(id, 'id不存在');
    try {
      await this.model.deleteOne({ _id: id });
      return { errcode: 0, errmsg: 'ok', data: '' };
    } catch (error) {
      throw error;
    }
  }
  async query({ skip, limit, name, code }) {
    const filter = {};
    const arr = { name, code };
    for (const e in arr) {
      const data = `{ "${e}": { "$regex": "${arr[e]}" } }`;
      if (arr[e]) {
        filter.$or = [];
        filter.$or.push(JSON.parse(data));
      }
    }
    try {
      const total = await this.model.find({ ...filter });
      let res;
      if (skip && limit) {
        res = await this.model.find({ ...filter }).skip(Number(skip) * Number(limit)).limit(Number(limit));
      } else {
        res = await this.model.find({ ...filter });
      }
      return { errcode: 0, errmsg: 'ok', data: res, total: total.length };
    } catch (error) {
      throw error;
    }
  }
}

module.exports = OrgService;
