'use strict';

const assert = require('assert');
const Service = require('egg').Service;
class OrgbindService extends Service {
  constructor(ctx) {
    super(ctx);
    this.model = this.ctx.model.Orgbind;
  }
  async create({ source, ids = [] }) {
    assert(source, 'source不存在');
    assert(ids, 'ids不存在');
    try {
      const res = await Promise.all(
        ids.filter(async e => {
          const info = await this.model.findOne({ source, target: e });
          if (!info) await this.model.create({ source, target: e });
        })
      );
      return { errcode: 0, errmsg: 'ok', data: res };
    } catch (error) {
      throw error;
    }
  }
  async update({ id, source, target }) {
    assert(id, 'id不存在');
    try {
      await this.model.updateOne({ _id: id }, { source, target });
      return { errcode: 0, errmsg: 'ok', data: '' };
    } catch (error) {
      throw error;
    }
  }
  async delete({ source, ids = [] }) {
    assert(ids, 'ids不存在');
    try {
      const res = await Promise.all(
        ids.filter(async e => {
          await this.model.deleteOne({ source, target: e });
        })
      );
      return { errcode: 0, errmsg: 'ok', data: res };
    } catch (error) {
      throw error;
    }
  }
  // 机构查询用户（聚合查询）
  async query({ skip, limit, source, target }) {
    const filter = {};
    const arr = { source, target };
    for (const e in arr) {
      const data = `{ "${e}": { "$regex": "${arr[e]}" } }`;
      if (arr[e]) {
        filter.$or = [];
        filter.$or.push(JSON.parse(data));
      }
    }
    try {
      const total = await this.model.find({ ...filter });
      let res;
      if (skip && limit) {
        res = await this.model.aggregate([
          {
            $match: { ...filter },
          },
          {
            $lookup: {
              // 要查询的表名
              from: 'user',
              // 左表字段
              localField: 'target',
              // 右表字段
              foreignField: 'openid',
              // 返回的json key
              as: 'userList',
            },
          },
          {
            $skip: +Number(skip * limit),
          },
          {
            $limit: Number(limit),
          },
        ]);
      } else {
        res = await this.model.aggregate([
          {
            $match: { ...filter },
          },
          {
            $lookup: {
              from: 'user',
              localField: 'target',
              foreignField: 'openid',
              as: 'userList',
            },
          },
        ]);
      }
      return { errcode: 0, errmsg: 'ok', data: res, total: total.length };
    } catch (error) {
      throw error;
    }
  }
}

module.exports = OrgbindService;
